@@ -10,6 +10,7 @@ import socket
 from async_property import async_cached_property
 from types import SimpleNamespace
 
+from neohubapi.enums import HCMode
 from neohubapi.enums import ScheduleFormat
 from neohubapi.enums import schedule_format_int_to_enum
 from neohubapi.neostat import NeoStat
@@ -64,7 +65,7 @@ class NeoHub:
                     return reply
                 if reply.__dict__ == expected_reply:
                     return True
-                self._logger.error(f"[{attempt}] Unexpected reply: {reply}")
+                self._logger.error(f"[{attempt}] Unexpected reply: {reply} for message: {message}")
             except (socket.gaierror, ConnectionRefusedError) as e:
                 last_exception = NeoHubConnectionError(e)
                 self._logger.error(f"[{attempt}] Could not connect to NeoHub at {self._host}: {e}")
@@ -84,6 +85,15 @@ class NeoHub:
             raise(last_exception)
         return False
 
+    def _devices_to_names(self, devices: [NeoStat]):
+        """
+            Returns the list of device names
+        """
+        try:
+            return [x.name for x in devices]
+        except (TypeError, AttributeError):
+            raise NeoHubUsageError('devices must be a list of NeoStat objects')
+
     async def firmware(self):
         """
         NeoHub firmware version
@@ -164,6 +174,17 @@ class NeoHub:
         result = await self._send(message, reply)
         return result
 
+    async def set_hc_mode(self, hc_mode: HCMode, devices: [NeoStat]):
+        """
+        Set hc_mode to AUTO or...
+        """
+        names = self._devices_to_names(devices)
+        message = {"SET_HC_MODE": [hc_mode.value, names]}
+        reply = {"result": "HC_MODE was set"}
+
+        result = await self._send(message, reply)
+        return result
+
     async def set_format(self, sched_format: ScheduleFormat):
         """
         Sets schedule format
@@ -424,11 +445,7 @@ class NeoHub:
             pin = pin // 10
         pins.reverse()
 
-        try:
-            names = [x.name for x in devices]
-        except (TypeError, AttributeError):
-            raise NeoHubUsageError('devices must be a list of NeoStat objects')
-
+        names = self._devices_to_names(devices)
         message = {"LOCK": [pins, names]}
         reply = {"result": "locked"}
 
@@ -440,11 +457,7 @@ class NeoHub:
         Unlocks PIN locked thermostats
         """
 
-        try:
-            names = [x.name for x in devices]
-        except (TypeError, AttributeError):
-            raise NeoHubUsageError('devices must be a list of NeoStat objects')
-
+        names = self._devices_to_names(devices)
         message = {"UNLOCK": names}
         reply = {"result": "unlocked"}
 
@@ -456,17 +469,29 @@ class NeoHub:
         Enables or disables Frost mode
         """
 
-        try:
-            names = [x.name for x in devices]
-        except (TypeError, AttributeError):
-            raise NeoHubUsageError('devices must be a list of NeoStat objects')
-
+        names = self._devices_to_names(devices)
         message = {"FROST_ON" if state else "FROST_OFF": names}
         reply = {"result": "frost on" if state else "frost off"}
 
         result = await self._send(message, reply)
         return result
 
+    async def set_cool_temp(self, temperature: int, devices: [NeoStat]):
+        """
+        Sets the thermostat's cooling temperature i.e. the temperature that will
+        trigger the thermostat if exceeded. Note that this is only supported on
+        the HC (Heating/Cooling) thermostats.
+
+        The temperature will be reset once next comfort level is reached
+        """
+
+        names = self._devices_to_names(devices)
+        message = {"SET_COOL_TEMP": [temperature, names]}
+        reply = {"result": "temperature was set"}
+
+        result = await self._send(message, reply)
+        return result
+
     async def set_target_temperature(self, temperature: int, devices: [NeoStat]):
         """
         Sets the thermostat's temperature
@@ -474,11 +499,7 @@ class NeoHub:
         The temperature will be reset once next comfort level is reached
         """
 
-        try:
-            names = [x.name for x in devices]
-        except (TypeError, AttributeError):
-            raise NeoHubUsageError('devices must be a list of NeoStat objects')
-
+        names = self._devices_to_names(devices)
         message = {"SET_TEMP": [temperature, names]}
         reply = {"result": "temperature was set"}
 
@@ -496,11 +517,7 @@ class NeoHub:
         3: 3 degrees
         """
 
-        try:
-            names = [x.name for x in devices]
-        except (TypeError, AttributeError):
-            raise NeoHubUsageError('devices must be a list of NeoStat objects')
-
+        names = self._devices_to_names(devices)
         message = {"SET_DIFF": [switching_differential, names]}
         reply = {"result": "switching differential was set"}
 
@@ -512,11 +529,7 @@ class NeoHub:
         Returns time in minutes required to change temperature by 1 degree
         """
 
-        try:
-            names = [x.name for x in devices]
-        except (TypeError, AttributeError):
-            raise NeoHubUsageError('devices must be a list of NeoStat objects')
-
+        names = self._devices_to_names(devices)
         message = {"VIEW_ROC": names}
 
         result = await self._send(message)
@@ -530,11 +543,7 @@ class NeoHub:
         NeoStats that are in timeclock mode.
         """
 
-        try:
-            names = [x.name for x in devices]
-        except (TypeError, AttributeError):
-            raise NeoHubUsageError('devices must be a list of NeoStat objects')
-
+        names = self._devices_to_names(devices)
         message = {"TIMER_ON" if state else "TIMER_OFF": names}
         reply = {"result": "timers on" if state else "timers off"}
 
@@ -548,11 +557,7 @@ class NeoHub:
         This function works with NeoStats in timeclock mode
         """
 
-        try:
-            names = [x.name for x in devices]
-        except (TypeError, AttributeError):
-            raise NeoHubUsageError('devices must be a list of NeoStat objects')
-
+        names = self._devices_to_names(devices)
         message = {"TIMER_HOLD_ON" if state else "TIMER_HOLD_OFF": [minutes, names]}
         reply = {"result": "timer hold on" if state else "timer hold off"}
 
